<*+CHANGESYM*>
<*+M2EXTENSIONS*>
<*+O2EXTENSIONS*>
<* ALIGNMENT = "8" *>
MODULE Sciter;
IMPORT 
  Windows, 
    SYSTEM, 
    ti     := Sciter_TIScriptAPI_Defs, 
    val    := Sciter_Value_Defs,       
    common := Sciter_Common_Defs,
    dom    := Sciter_DOM_Defs;

TYPE
  HELEMENT* = dom.HELEMENT;
  SCDOM_RESULT* = dom.SCDOM_RESULT;
 
  THTMLAYOUT_NOTIFY* = PROCEDURE ["StdCall"] (uMsg: Windows.UINT; wParam: Windows.WPARAM; lParam: Windows.LPARAM; vParam: Windows.PVOID): Windows.LRESULT;

  PSCITER_VALUE* = POINTER ["StdCall"] TO SCITER_VALUE;
  SCITER_VALUE* = val.VALUE;

  PSCN_LOAD_DATA* = POINTER ["StdCall"] TO SCN_LOAD_DATA;
  SCN_LOAD_DATA* = RECORD  (common.SCITER_CALLBACK_NOTIFICATION)
    uri*: Windows.PCWSTR;       -- < [in] Zero terminated string, fully qualified uri, for example "http://server/folder/file.ext".
    outData*:     Windows.PCBYTE;   -- < [in,out] pointer to loaded data to return. if data exists in the cache then this field contain pointer to it*/
    outDataSize*: Windows.UINT; -- < [in,out] loaded data size to return.*/
    dataType*:    Windows.UINT;    -- < [in] SciterResourceType */                              
    requestId*:   Windows.PVOID;  -- < [in] request id that needs to be passed as is to the SciterDataReadyAsync call */
    principal*:   HELEMENT;
    initiator*:   HELEMENT;
  END;

  PSCN_DATA_LOADED* = POINTER ["StdCall"] TO SCN_DATA_LOADED;
  SCN_DATA_LOADED* = RECORD (common.SCITER_CALLBACK_NOTIFICATION)
    uri*:      Windows.PCWSTR; -- < [in] zero terminated string, fully qualified uri, for example "http://server/folder/file.ext".
    data*:     Windows.PCBYTE; -- < [in] pointer to loaded data.
    dataSize*: Windows.DWORD;  -- < [in] loaded data size (in bytes).
    dataType*: Windows.UINT;   -- < [in] SciterResourceType 
    status*:   Windows.UINT;   -- < [in] status = 0 (dataSize == 0) - unknown error. 
                               --        status = 100..505 - http response status, Note: 200 - OK! 
                               --        status > 12000 - wininet error code, see ERROR_INTERNET_*** in wininet.h                                   
  END;

  PSCN_ATTACH_BEHAVIOR* = POINTER ["StdCall"] TO SCN_ATTACH_BEHAVIOR;
  SCN_ATTACH_BEHAVIOR* = RECORD(common.SCITER_CALLBACK_NOTIFICATION)
    element*:      HELEMENT;         -- < [in] target DOM element handle
    behaviorName*: Windows.PCSTR;    -- < [in] zero terminated string, string appears as value of CSS behavior:"???" attribute.
    elementProc*:  common.ElementEventProc; -- < [out] pointer to ElementEventProc function.
    elementTag*:   Windows.PVOID;    -- < [out] tag value, passed as is into pointer ElementEventProc function.
  END;

  PSCN_CALLBACK_HOST* = POINTER ["StdCall"] TO SCN_CALLBACK_HOST;
  SCN_CALLBACK_HOST* = RECORD(common.SCITER_CALLBACK_NOTIFICATION)
    channel*: Windows.UINT; -- 0 - stdin, 1 - stdout, 2 - stderr
    p1*:      SCITER_VALUE; -- in, parameter #1
    p2*:      SCITER_VALUE; -- in, parameter #2
    r*:       SCITER_VALUE; -- out, retval
  END;

CONST
  LOAD_OK*      = 0; -- do default loading if data not set
  LOAD_DISCARD* = 1; -- discard request completely
  LOAD_DELAYED* = 2; -- data will be delivered later by the host

  SC_LOAD_DATA*         = 01H;

(**This notification indicates that external data (for example image) download process
 * completed.
 *
 * param lParam #LPSCN_DATA_LOADED
 *
 * This notifiaction is sent for each external resource used by document when
 * this resource has been completely downloaded. Sciter will send this
 * notification asynchronously.
 **)
  SC_DATA_LOADED*       = 02H;

(**This notification is sent when all external data (for example image) has been downloaded.
 *
 * This notification is sent when all external resources required by document
 * have been completely downloaded. Sciter will send this notification
 * asynchronously.
 **)
  SC_DOCUMENT_COMPLETE* = 03H;

(**This notification is sent on parsing the document and while processing
 * elements having non empty style.behavior attribute value.
 *
 * param lParam #LPSCN_ATTACH_BEHAVIOR
 *
 * Application has to provide implementation of #sciter::behavior interface.
 * Set #SCN_ATTACH_BEHAVIOR::impl to address of this implementation.
 **)
  SC_ATTACH_BEHAVIOR* = 04H;

(**This notification is sent on
 * 1) stdin, stdout and stderr operations and
 * 2) view.hostCallback(p1,p2) calls from script
 *
 * param lParam #LPSCN_CALLBACK_HOST
 *
 **)
  SC_CALLBACK_HOST* = 05H;

TYPE
  SciterHostCallback* = PROCEDURE ["StdCall"] (pns: Windows.PVOID; callbackParam: Windows.PVOID): Windows.UINT;

TYPE
  HVM* = ti.Ptiscript_VM;

  SciterNativeMethod_t* = PROCEDURE ["StdCall"] (hvm: HVM; VAR selfp: SCITER_VALUE; VAR argv: SCITER_VALUE; argc: SYSTEM.INT32; VAR retval: SCITER_VALUE); 
  SciterNativeProperty_t* = PROCEDURE ["StdCall"] (hvm: HVM; VAR selfp: SCITER_VALUE; sets: SYSTEM.INT32; VAR value: SCITER_VALUE);
  SciterNativeDtor_t* = PROCEDURE ["StdCall"] (hvm: HVM; p_data_slot_value: SYSTEM.ADDRESS);

  PSciterNativeMethodDef* = POINTER ["StdCall"] TO SciterNativeMethodDef;
  SciterNativeMethodDef*  = RECORD  ["StdCall"]
      name*: Windows.PCSTR;
      method*: SciterNativeMethod_t;
  END;

  PSciterNativePropertyDef* = POINTER ["StdCall"] TO SciterNativePropertyDef;
  SciterNativePropertyDef*  = RECORD  
      name*: Windows.PCSTR;
      propertys*: SciterNativeProperty_t;
  END;

  PSciterNativeConstantDef* = POINTER ["StdCall"] TO SciterNativeConstantDef;
  SciterNativeConstantDef*  = RECORD  
      name*: Windows.PCSTR;
      value*: SCITER_VALUE;
--      SciterNativeConstantDef( LPCSTR n,SCITER_VALUE v = SCITER_VALUE() ):name(n),value(v) {}
  END;

  PSciterNativeClassDef* = POINTER ["StdCall"] TO SciterNativeClassDef;
  SciterNativeClassDef*  = RECORD  
      name*: Windows.PCSTR;
      methods*: PSciterNativeMethodDef;
      properties*: PSciterNativePropertyDef;
      dtor*: SciterNativeDtor_t;
      constants*: PSciterNativeConstantDef;
  END;

  PROCEDURE ["StdCall"] / SciterClassNameA* (): Windows.PCSTR;
  PROCEDURE ["StdCall"] / SciterClassNameW* (): Windows.PCWSTR;
  PROCEDURE ["StdCall"] / SciterGetVM* (handle: Windows.HWND): HVM;
  PROCEDURE ["StdCall"] / SciterNativeDefineClass* ( hvm: HVM; pClassDef: PSciterNativeClassDef): SYSTEM.INT32;
  PROCEDURE ["StdCall"] / SciterSetExceptionValue* ( hvm: HVM; errorMsg: Windows.PCWSTR): SYSTEM.INT32;
  PROCEDURE ["StdCall"] / SciterVmEval* ( hvm: HVM; script: Windows.PCWSTR; scriptLength: Windows.UINT; VAR retval: SCITER_VALUE): SYSTEM.INT32;
  PROCEDURE ["StdCall"] / SciterCreateObject* (hvm: HVM; className: Windows.PCWSTR; VAR retval: SCITER_VALUE): SYSTEM.INT32;

-- Calls method 'function' of the object 'obj' with parameters given as VALUEs rather tiscript_value. 
-- This method can be used in inter-thread/inter-vm communications. 
  PROCEDURE ["StdCall"] / SciterCall_V*(vm: HVM; obj: ti.tiscript_value; methodName: Windows.PSTR; 
                                       VAR argv: SCITER_VALUE; argn: Windows.UINT; VAR retval: SCITER_VALUE): Windows.BOOL;

-- Converts tiscript value handle to the json::value (VALUE) with optional isolation.
-- tiscript::value handle is a handle that specific for particular VM
-- json::value can be passed across different VMs, threads and can be stored outside VM heap.
-- isolate==true - will create value that can be passed across thread and VM boundaries.
  PROCEDURE ["StdCall"] / Sciter_v2V*(vm: HVM; tisv: ti.tiscript_value; VAR jsonv: SCITER_VALUE; isolate: Windows.BOOL): Windows.BOOL;

-- Backward conversion
  PROCEDURE ["StdCall"] / Sciter_V2v*(vm: HVM; VAR jsonv: SCITER_VALUE; VAR tisv: ti.tiscript_value): Windows.BOOL;


(* SCITE Section *)

(* Native API *)
TYPE
  tiscript_native_interface* = ti.tiscript_native_interface;
  Ptiscript_native_interface* = ti.Ptiscript_native_interface;

  PROCEDURE ["StdCall"] / TIScriptAPI*(): Ptiscript_native_interface;

TYPE 
  SCITER_RT_OPTIONS* = SYSTEM.INT32;

CONST
   SCITER_SMOOTH_SCROLL*      = 1; -- value:TRUE - enable, value:FALSE - disable, enabled by default
   SCITER_CONNECTION_TIMEOUT* = 2; -- value: milliseconds, connection timeout of http client
   SCITER_HTTPS_ERROR*        = 3; -- value: 0 - drop connection, 1 - use builtin dialog, 2 - accept connection silently
   SCITER_FONT_SMOOTHING*     = 4; -- value: 0 - system default, 1 - no smoothing, 2 - std smoothing, 3 - clear type
   SCITER_TRANSPARENT_WINDOW* = 6; -- Windows Aero support, value: 
                                   -- 0 - normal drawing, 
                                   -- 1 - window has transparent background after calls DwmExtendFrameIntoClientArea() or DwmEnableBlurBehindWindow().

  PROCEDURE ["StdCall"] / SciterVersion*(major: Windows.BOOL): Windows.UINT;
  PROCEDURE ["StdCall"] / SciterDataReady*(hWnd: Windows.HWND; uri: Windows.PWSTR; data: Windows.PBYTE; dataLength: Windows.DWORD): Windows.BOOL;
  PROCEDURE ["StdCall"] / SciterDataReadyAsync*(hWnd: Windows.HWND; uri: Windows.PWSTR; data: Windows.PBYTE; dataLength: Windows.DWORD; requestId: Windows.PVOID): Windows.BOOL;
  PROCEDURE ["StdCall"] / SciterResetURLData*(hWnd: Windows.HWND; uri: Windows.PWSTR; data: Windows.PBYTE; dataLength: Windows.DWORD; dataType: Windows.UINT): Windows.BOOL;
  PROCEDURE ["StdCall"] / SciterProc*(hWnd: Windows.HWND; msg: Windows.UINT; wParam: Windows.WPARAM; lParam: Windows.LPARAM): Windows.LRESULT;
  PROCEDURE ["StdCall"] / SciterProcND*(hWnd: Windows.HWND; msg: Windows.UINT; wParam: Windows.WPARAM; lParam: Windows.LPARAM; VAR bHandled: Windows.BOOL): Windows.LRESULT;
  PROCEDURE ["StdCall"] / SciterLoadFile*(hWndSciter: Windows.HWND; filename: Windows.PWSTR): Windows.BOOL;
  PROCEDURE ["StdCall"] / SciterLoadHtml*(hWndSciter: Windows.HWND; html: Windows.PBYTE; htmlSize: Windows.UINT; baseUrl: Windows.PWSTR): Windows.BOOL;
  PROCEDURE ["StdCall"] / SciterSetCallback*(hWndSciter: Windows.HWND; cb: SciterHostCallback; cbParam: Windows.PVOID);
  PROCEDURE ["StdCall"] / SciterSetMasterCSS*(utf8: Windows.PBYTE; numBytes: Windows.UINT): Windows.BOOL;
  PROCEDURE ["StdCall"] / SciterAppendMasterCSS*(utf8: Windows.PBYTE; numBytes: Windows.UINT): Windows.BOOL;
  PROCEDURE ["StdCall"] / SciterSetCSS*(hWndSciter: Windows.HWND; utf8: Windows.PBYTE; numBytes: Windows.UINT; baseUrl: Windows.PWSTR; mediaType: Windows.PWSTR): Windows.BOOL;
  PROCEDURE ["StdCall"] / SciterSetMediaType*(hWndSciter: Windows.HWND; mediaType: Windows.PWSTR): Windows.BOOL;
  PROCEDURE ["StdCall"] / SciterSetMediaVars*(hWndSciter: Windows.HWND; VAR mediaVars: SCITER_VALUE): Windows.BOOL;
  PROCEDURE ["StdCall"] / SciterGetMinWidth*(hWndSciter: Windows.HWND): Windows.UINT;
  PROCEDURE ["StdCall"] / SciterGetMinHeight*(hWndSciter: Windows.HWND; width: Windows.UINT): Windows.UINT;
  PROCEDURE ["StdCall"] / SciterCall*(hWnd: Windows.HWND; functionName: Windows.PCSTR; argc: Windows.UINT; VAR argv: SCITER_VALUE; VAR retval: SCITER_VALUE): Windows.BOOL;
  PROCEDURE ["StdCall"] / SciterEval*(hWnd: Windows.HWND; script: Windows.PCWSTR; scriptLength: Windows.UINT; VAR retval: SCITER_VALUE): Windows.BOOL;
  PROCEDURE ["StdCall"] / SciterUpdateWindow*(hwnd: Windows.HWND);
  PROCEDURE ["StdCall"] / SciterTranslateMessage*(VAR Msg: Windows.MSG): Windows.BOOL;
  PROCEDURE ["StdCall"] / SciterSetOption*(hWnd: Windows.HWND; option: SCITER_RT_OPTIONS; value: Windows.UINT): Windows.BOOL;
  PROCEDURE ["StdCall"] / SciterRender*(hWndSciter: Windows.HWND; hBmp: Windows.HBITMAP; area: Windows.RECT): Windows.BOOL;
  PROCEDURE ["StdCall"] / SciterSetHomeURL*(hWndSciter: Windows.HWND; baseUrl: Windows.PCWSTR): Windows.BOOL;

TYPE
  DEBUG_OUTPUT_PROC = PROCEDURE  ["StdCall"] (param: Windows.PVOID; character: Windows.LONG);

PROCEDURE ["StdCall"] / SciterSetupDebugOutput*(param: Windows.PVOID; fOutput: DEBUG_OUTPUT_PROC);

END Sciter.