<*+CHANGESYM*>
<*+M2EXTENSIONS*>
<*+O2EXTENSIONS*>
<* ALIGNMENT = "8" *>
MODULE Sciter_Behavior_Defs;

IMPORT 
  Windows, 
  ti     := Sciter_TIScriptAPI_Defs,
  dom    := Sciter_DOM_Defs,
  common := Sciter_Common_Defs,
  val    := Sciter_Value_Defs,
  SYSTEM;

TYPE 
  EVENT_GROUPS* = common.EVENT_GROUPS;

CONST 
  HANDLE_INITIALIZATION        * = EVENT_GROUPS     { }; -- =         0H; (* attached/detached  *)
  HANDLE_MOUSE                 * = EVENT_GROUPS     {0}; -- =         1H; (* mouse events  *)
  HANDLE_KEY                   * = EVENT_GROUPS     {1}; -- =         2H; (* key events  *)
  HANDLE_FOCUS                 * = EVENT_GROUPS     {2}; -- =         4H; (* focus events, if this flag is set it also means that element it attached to is focusable  *)
  HANDLE_SCROLL                * = EVENT_GROUPS     {3}; -- =         8H; (* scroll events  *)
  HANDLE_TIMER                 * = EVENT_GROUPS     {4}; -- =        10H; (* timer event  *)
  HANDLE_SIZE                  * = EVENT_GROUPS     {5}; -- =        20H; (* size changed event  *)
  HANDLE_DRAW                  * = EVENT_GROUPS     {6}; -- =        40H; (* drawing request (event)  *)
  HANDLE_DATA_ARRIVED          * = EVENT_GROUPS     {7}; -- =        80H; (* requested data () has been delivered  *)
  HANDLE_BEHAVIOR_EVENT        * = EVENT_GROUPS     {8}; -- =       100H; (* logical, synthetic events: BUTTON_CLICK, HYPERLINK_CLICK, etc., a.k.a. notifications from intrinsic behaviors *)
  HANDLE_METHOD_CALL           * = EVENT_GROUPS     {9}; -- =       200H; (* behavior specific methods  *)
  HANDLE_SCRIPTING_METHOD_CALL * = EVENT_GROUPS    {10}; -- =       400H; (* behavior specific methods  *)
  HANDLE_TISCRIPT_METHOD_CALL  * = EVENT_GROUPS    {11}; -- =       800H; (* behavior specific methods using direct tiscript::value's  *)
  HANDLE_EXCHANGE              * = EVENT_GROUPS    {12}; -- =      1000H; (* system drag-n-drop  *)
  HANDLE_GESTURE               * = EVENT_GROUPS    {13}; -- =      2000H; (* touch input events  *)
  HANDLE_ALL                   * = EVENT_GROUPS {0..15}; -- =     0FFFFH; (* all of them  *)
  SUBSCRIPTIONS_REQUEST        * = EVENT_GROUPS {0..31}; -- = 0FFFFFFFFH; (* special value for getting subscription flags  *)

TYPE
  EVENT_MASK* = SYSTEM.SET32;


CONST 
  BUBBLING* = EVENT_MASK   {};            (*  bubbling (emersion) phase *)
  SINKING * = EVENT_MASK {15}; --  8000H; (*  capture (immersion) phase, this flag is or'ed with EVENTS codes below *)
  HANDLED * = EVENT_MASK {16}; -- 10000H;


TYPE 
  PHASE_MASK* = EVENT_MASK;

CONST 
  MAIN_MOUSE_BUTTON  * = EVENT_MASK {0}; (* aka left button *)
  PROP_MOUSE_BUTTON  * = EVENT_MASK {1}; (* aka right button *)
  MIDDLE_MOUSE_BUTTON* = EVENT_MASK {2};

TYPE 
  MOUSE_BUTTONS* = EVENT_MASK;

CONST 
  CONTROL_KEY_PRESSED* = EVENT_MASK {0}; -- 1H;
  SHIFT_KEY_PRESSED  * = EVENT_MASK {1}; -- 2H;
  ALT_KEY_PRESSED    * = EVENT_MASK {2}; -- 4H;

TYPE 
  KEYBOARD_STATES* = EVENT_MASK;

CONST
  BEHAVIOR_DETACH* = 0;
  BEHAVIOR_ATTACH* = 1;


TYPE 
  INITIALIZATION_EVENTS* = EVENT_MASK;

CONST 
  NO_DRAGGING  * = 0;
  DRAGGING_MOVE* = 1;
  DRAGGING_COPY* = 2;

TYPE 
  DRAGGING_TYPE* = Windows.UINT;


CONST 
  MOUSE_ENTER * =    0;
  MOUSE_LEAVE * =    1;
  MOUSE_MOVE  * =    2;
  MOUSE_UP    * =    3;
  MOUSE_DOWN  * =    4;
  MOUSE_DCLICK* =    5;
  MOUSE_WHEEL * =    6;
  MOUSE_TICK  * =    7; (*  mouse pressed ticks *)
  MOUSE_IDLE  * =    8; (*  mouse stay idle for some time *)
  DROP        * =    9; (*  item dropped, target is that dropped item  *)
  DRAG_ENTER  * =  0AH; (*  drag arrived to the target element that is one of current drop targets.   *)
  DRAG_LEAVE  * =  0BH; (*  drag left one of current drop targets. target is the drop target element.   *)
  DRAG_REQUEST* =  0CH; (*  drag src notification before drag start. To cancel - return true from handler. *)
  MOUSE_CLICK * = 0FFH; (*  mouse click event *)
  DRAGGING    * = 100H; (*  This flag is 'ORed' with MOUSE_ENTER..MOUSE_DOWN codes if dragging operation is in effect. *)


(*  E.g. event DRAGGING | MOUSE_MOVE is sent to underlying DOM elements while dragging. *)

TYPE 
  MOUSE_EVENTS* = Windows.UINT;

CONST 
  CURSOR_ARROW      * =  0;    (* 0 *)
  CURSOR_IBEAM      * =  1;    (* 1 *)
  CURSOR_WAIT       * =  2;    (* 2 *)
  CURSOR_CROSS      * =  3;    (* 3 *)
  CURSOR_UPARROW    * =  4;    (* 4 *)
  CURSOR_SIZENWSE   * =  5;    (* 5 *)
  CURSOR_SIZENESW   * =  6;    (* 6 *)
  CURSOR_SIZEWE     * =  7;    (* 7 *)
  CURSOR_SIZENS     * =  8;    (* 8 *)
  CURSOR_SIZEALL    * =  9;    (* 9 *)
  CURSOR_NO         * = 10;   (* 10 *)
  CURSOR_APPSTARTING* = 11;   (* 11 *)
  CURSOR_HELP       * = 12;   (* 12 *)
  CURSOR_HAND       * = 13;   (* 13 *)
  CURSOR_DRAG_MOVE  * = 14;   (* 14 *)
  CURSOR_DRAG_COPY  * = 15;   (* 15 *)

TYPE 
  CURSOR_TYPE* = Windows.UINT;

CONST 
  KEY_DOWN* = 0;
  KEY_UP  * = 1;
  KEY_CHAR* = 2;

TYPE 
  KEY_EVENTS* = Windows.UINT;

CONST 
  FOCUS_LOST* = 0;
  FOCUS_GOT * = 1;

TYPE 
  FOCUS_EVENTS* = Windows.UINT;

CONST 
  SCROLL_HOME           * = 0;
  SCROLL_END            * = 1;
  SCROLL_STEP_PLUS      * = 2;
  SCROLL_STEP_MINUS     * = 3;
  SCROLL_PAGE_PLUS      * = 4;
  SCROLL_PAGE_MINUS     * = 5;
  SCROLL_POS            * = 6;
  SCROLL_SLIDER_RELEASED* = 7;
  SCROLL_CORNER_PRESSED * = 8;
  SCROLL_CORNER_RELEASED* = 9;

TYPE 
  SCROLL_EVENTS* = Windows.UINT;

CONST 
  GESTURE_REQUEST* = 0; (*  return true and fill flags if it will handle gestures. *)
  GESTURE_ZOOM   * = 1; (*  The zoom gesture. *)
  GESTURE_PAN    * = 2; (*  The pan gesture. *)
  GESTURE_ROTATE * = 3; (*  The rotation gesture. *)
  GESTURE_TAP1   * = 4; (*  The tap gesture. *)
  GESTURE_TAP2   * = 5; (*  The two-finger tap gesture. *)

TYPE 
  GESTURE_CMD* = Windows.UINT;

CONST 
  GESTURE_STATE_BEGIN  * = EVENT_MASK {0}; (* starts *)
  GESTURE_STATE_INERTIA* = EVENT_MASK {1}; (* events generated by inertia processor *)
  GESTURE_STATE_END    * = EVENT_MASK {2}; (* end, last event of the gesture sequence *)

TYPE 
  GESTURE_STATE* = EVENT_MASK;

CONST 
  GESTURE_FLAG_ZOOM            * = EVENT_MASK {0};     -- =     1H;
  GESTURE_FLAG_ROTATE          * = EVENT_MASK {1};     -- =     2H;
  GESTURE_FLAG_PAN_VERTICAL    * = EVENT_MASK {2};     -- =     4H;
  GESTURE_FLAG_PAN_HORIZONTAL  * = EVENT_MASK {3};     -- =     8H;
  GESTURE_FLAG_TAP1            * = EVENT_MASK {4};     -- =    10H; (* press & tap *)
  GESTURE_FLAG_TAP2            * = EVENT_MASK {5};     -- =    20H; (* two fingers tap *)
  GESTURE_FLAG_PAN_WITH_GUTTER * = EVENT_MASK {15};    -- =  4000H; (* PAN_VERTICAL and PAN_HORIZONTAL modifiers *)
  GESTURE_FLAG_PAN_WITH_INERTIA* = EVENT_MASK {15};    -- =  8000H;
  GESTURE_FLAGS_ALL            * = EVENT_MASK {0..15}; -- = 0FFFFH;

TYPE 
  GESTURE_TYPE_FLAGS* = EVENT_MASK;

CONST 
  DRAW_BACKGROUND* = 0;
  DRAW_CONTENT   * = 1;
  DRAW_FOREGROUND* = 2;

TYPE 
  DRAW_EVENTS* = Windows.UINT;

CONST 
  BUTTON_CLICK                * =   0;   (*  click on button *)
  BUTTON_PRESS                * =   1;   (*  mouse down or key down in button *)
  BUTTON_STATE_CHANGED        * =   2;   (*  checkbox/radio/slider changed its state/value *)
  EDIT_VALUE_CHANGING         * =   3;   (*  before text change *)
  EDIT_VALUE_CHANGED          * =   4;   (*  after text change *)
  SELECT_SELECTION_CHANGED    * =   5;   (*  selection in <select> changed *)
  SELECT_STATE_CHANGED        * =   6;   (*  node in select expanded/collapsed, heTarget is the node *)
  POPUP_REQUEST               * =   7;   (*  request to show popup just received, here DOM of popup element can be modifed. *)
  POPUP_READY                 * =   8;   (*  popup element has been measured and ready to be shown on screen, *)
  POPUP_DISMISSED             * =   9;   (*  popup element is closed, here DOM of popup element can be modifed again - e.g. some items can be removed to free memory. *)
  MENU_ITEM_ACTIVE            * = 0AH;   (*  menu item activated by mouse hover or by keyboard, *)
  MENU_ITEM_CLICK             * = 0BH;   (*  menu item click, 
                                              BEHAVIOR_EVENT_PARAMS structure layout 
                                              BEHAVIOR_EVENT_PARAMS.cmd - MENU_ITEM_CLICK/MENU_ITEM_ACTIVE 
                                              BEHAVIOR_EVENT_PARAMS.heTarget - owner(anchor) of the menu 
                                              BEHAVIOR_EVENT_PARAMS.he - the menu item, presumably <li> element 
                                              BEHAVIOR_EVENT_PARAMS.reason - BY_MOUSE_CLICK | BY_KEY_CLICK *)
  CONTEXT_MENU_REQUEST        * = 10H;   (*  "right-click", BEHAVIOR_EVENT_PARAMS::he is current popup menu HELEMENT being processed or NULL. 
                                             application can provide its own HELEMENT here (if it is NULL) or modify current menu element. *)
  VISIUAL_STATUS_CHANGED      * = 11H;   (*  broadcast notification, sent to all elements of some container being shown or hidden *)
  DISABLED_STATUS_CHANGED     * = 12H;   (*  broadcast notification, sent to all elements of some container that got new value of :disabled state *)
  POPUP_DISMISSING            * = 13H;   (*  popup is about to be closed "grey" event codes  - notfications from behaviors from this SDK *)
  HYPERLINK_CLICK             * = 80H;   (*  hyperlink click *)
  TABLE_HEADER_CLICK          * = 129;   (*  click on some cell in table header, target                        = the cell, reason = index of the cell (column number, 0..n) *)
  TABLE_ROW_CLICK             * = 130;   (*  click on data row in the table, target is the row target          = the row, reason  = index of the row (fixed_rows..n) *)
  TABLE_ROW_DBL_CLICK         * = 131;   (*  mouse dbl click on data row in the table, target is the row target= the row, reason  = index of the row (fixed_rows..n) *)
  ELEMENT_COLLAPSED           * = 90H;   (*  element was collapsed, so far only behavior:tabs is sending these two to the panels *)
  ELEMENT_EXPANDED            * = 145;   (*  element was expanded, *)
  ACTIVATE_CHILD              * = 146;   (*  activate (select) child, used for example by accesskeys behaviors to send activation request, e.g. tab on behavior:tabs. *)
  DO_SWITCH_TAB               * = ACTIVATE_CHILD; (*  command to switch tab programmatically, handled by behavior:tabs 
                                            use it as HTMLayoutPostEvent(tabsElementOrItsChild, DO_SWITCH_TAB, tabElementToShow, 0); *)
  INIT_DATA_VIEW              * = 147;   (*  request to virtual grid to initialize its view *)
  ROWS_DATA_REQUEST           * = 148;   (*  request from virtual grid to data source behavior to fill data in the table parameters passed throug DATA_ROWS_PARAMS structure. *)
  UI_STATE_CHANGED            * = 149;   (*  ui state changed, observers shall update their visual states. 
                                            is sent for example by behavior:richtext when caret position/selection has changed. *)
  FORM_SUBMIT                 * = 150;   (*  behavior:form detected submission event. BEHAVIOR_EVENT_PARAMS::data field contains data to be posted. 
                                            BEHAVIOR_EVENT_PARAMS::data is of type T_MAP in this case key/value pairs of data that is about  
                                            to be submitted. You can modify the data or discard submission by returning TRUE from the handler. *)
  FORM_RESET                  * = 151;   (*  behavior:form detected reset event (from button type = reset). 
                                              BEHAVIOR_EVENT_PARAMS::data field contains data to be reset. 
                                              BEHAVIOR_EVENT_PARAMS::data is of type T_MAP in this case key/value pairs of data that is about  
                                              to be rest. You can modify the data or discard reset by returning TRUE from the handler. *)
  DOCUMENT_COMPLETE           * = 152;   (*  document in behavior:frame or root document is complete. *)
  HISTORY_PUSH                * = 153;   (*  requests to behavior:history (commands) *)
  HISTORY_DROP                * = 154;
  HISTORY_PRIOR               * = 155;
  HISTORY_NEXT                * = 156;
  HISTORY_STATE_CHANGED       * = 157;   (*  behavior:history notification - history stack has changed *)
  CLOSE_POPUP                 * = 158;   (*  close popup request, *)
  REQUEST_TOOLTIP             * = 159;   (*  request tooltip, evt.source <- is the tooltip element. *)
  ANIMATION                   * = 0A0H;  (*  animation started (reason = 1) or ended(reason=0) on the element. *)
  FIRST_APPLICATION_EVENT_CODE* = 100H;  (* all custom event codes shall be greater 
                                           than this number. All codes below this will be used 
                                           solely by application - HTMLayout will not intrepret it 
                                           and will do just dispatching. 
                                           To send event notifications with  these codes use 
                                           HTMLayoutSend/PostEvent API. *)

TYPE 
  BEHAVIOR_EVENTS* = Windows.UINT;

CONST 
  BY_MOUSE_CLICK* = 0;
  BY_KEY_CLICK  * = 1;
  SYNTHESIZED   * = 2; (* synthesized, programmatically generated. *)

TYPE 
  EVENT_REASON* = Windows.UINT;

CONST 
  BY_INS_CHAR * = 0; (* single char insertion *)
  BY_INS_CHARS* = 1; (* character range insertion, clipboard *)
  BY_DEL_CHAR * = 2; (* single char deletion *)
  BY_DEL_CHARS* = 3; (* character range deletion (selection) *)

TYPE 
  EDIT_CHANGED_REASON* = Windows.UINT;

CONST 
  DO_CLICK                    * =    0;
  GET_TEXT_VALUE              * =    1;
  SET_TEXT_VALUE              * =    2; (*  p - TEXT_VALUE_PARAMS *)
  TEXT_EDIT_GET_SELECTION     * =    3; (*  p - TEXT_EDIT_SELECTION_PARAMS *)
  TEXT_EDIT_SET_SELECTION     * =    4; (*  p - TEXT_EDIT_SELECTION_PARAMS 
                                            Replace selection content or insert text at current caret position. 
                                            Replaced text will be selected. *)
  TEXT_EDIT_REPLACE_SELECTION * =    5; (*  p - TEXT_EDIT_REPLACE_SELECTION_PARAMS 
                                            Set value of type ="vscrollbar"/"hscrollbar" *)
  SCROLL_BAR_GET_VALUE        * =    6;
  SCROLL_BAR_SET_VALUE        * =    7;
  TEXT_EDIT_GET_CARET_POSITION* =    8;
  TEXT_EDIT_GET_SELECTION_TEXT* =    9; (*  p - TEXT_SELECTION_PARAMS *)
  TEXT_EDIT_GET_SELECTION_HTML* =   10; (*  p - TEXT_SELECTION_PARAMS *)
  TEXT_EDIT_CHAR_POS_AT_XY    * =   11; (*  p - TEXT_EDIT_CHAR_POS_AT_XY_PARAMS *)
  IS_EMPTY                    * = 0FCH; (*  p - IS_EMPTY_PARAMS // set VALUE_PARAMS::is_empty (false/true) reflects :empty 
                                            state of the element. *)
  GET_VALUE                   * = 0FDH; (*  p - VALUE_PARAMS  *)
  SET_VALUE                   * = 0FEH; (*  p - VALUE_PARAMS  *)
  FIRST_APPLICATION_METHOD_ID * = 100H;

TYPE 
  BEHAVIOR_METHOD_IDENTIFIERS* = Windows.UINT;

TYPE
  PARAMS* = common.PARAMS;
  PPARAMS* = common.PPARAMS;

  INITIALIZATION_PARAMS* = RECORD (PARAMS)
    cmd*: INITIALIZATION_EVENTS; -- INITIALIZATION_EVENTS  
  END;

  MOUSE_PARAMS* = RECORD (PARAMS)
    cmd*: MOUSE_EVENTS;             -- MOUSE_EVENTS 
    target*: dom.HELEMENT;          -- target element
    pos*: Windows.POINT;            -- position of cursor, element relative
    pos_view*: Windows.POINT;       -- position of cursor, view relative
    button_state*: MOUSE_BUTTONS;   
    alt_state*: KEYBOARD_STATES;
    cursor_type*: CURSOR_TYPE;      --  to set, see CURSOR_TYPE
    is_on_icon*: Windows.BOOL;     -- mouse is over icon (foreground-image, foreground-repeat:no-repeat)
    dragging*: dom.HELEMENT;        -- element that is being dragged over, this field is not NULL if (cmd & DRAGGING) != 0
    dragging_mode*: DRAGGING_TYPE;
  END;

  KEY_PARAMS* = RECORD(PARAMS)
    cmd*: KEY_EVENTS;
    target*: dom.HELEMENT;       -- target element
    key_code*: Windows.UINT;     -- key scan code, or character unicode for KEY_CHAR
    alt_state*: KEYBOARD_STATES;
  END;

  FOCUS_PARAMS* = RECORD(PARAMS)
    cmd*: FOCUS_EVENTS;
    target*: dom.HELEMENT;          -- target element, for FOCUS_LOST it is a handle of new focus element
                                    -- and for FOCUS_GOT it is a handle of old focus element, can be NULL
		by_mouse_click*: Windows.BOOL; -- TRUE if focus is being set by mouse click
		cancel*: Windows.BOOL;         -- in FOCUS_LOST phase setting this field to TRUE will cancel transfer focus from old element to the new one.
	END;

  SCROLL_PARAMS* = RECORD
    cmd*: SCROLL_EVENTS;
    target*: dom.HELEMENT;    -- target element                 
    pos*: Windows.LONG;       -- scroll position if SCROLL_POS  
    vertical*: Windows.BOOL; -- TRUE if from vertical scrollbar
  END;

  GESTURE_PARAMS* = RECORD(PARAMS)
    cmd*: GESTURE_CMD;
    target*: dom.HELEMENT;      -- target element
    pos*: Windows.POINT;        -- position of cursor, element relative
    pos_view*: Windows.POINT;   -- position of cursor, view relative
    flags*: GESTURE_TYPE_FLAGS; -- for GESTURE_REQUEST combination of GESTURE_FLAGs.
                                -- for others it is a combination of GESTURE_STATe's
    delta_time*: Windows.UINT;  -- period of time from previous event.
    delta_xy*: Windows.SIZEL;   -- for GESTURE_PAN it is a direction vector
    delta_v*: LONGREAL;         -- for GESTURE_ROTATE - delta angle (radians)
                               -- for GESTURE_ZOOM - zoom value, is less or greater than 1.0
  END;

  DRAW_PARAMS* = RECORD(PARAMS)
    cmd*: DRAW_EVENTS;          
    hdc*: Windows.HDC;           -- hdc to paint on
    area*: Windows.RECT;         -- element area, to get invalid area to paint use GetClipBox,
    reserved*: Windows.UINT;     -- for DRAW_BACKGROUND/DRAW_FOREGROUND - it is a border box

    -- Use ::GetTextColor(hdc) to get current text color of the element
  END;

  BEHAVIOR_EVENT_PARAMS* = RECORD(PARAMS)
    cmd*: BEHAVIOR_EVENTS; 
    heTarget*: dom.HELEMENT; -- target element handler, in MENU_ITEM_CLICK this is owner element that caused this menu - e.g. context menu owner
                             -- In scripting this field named as Event.owner
    he*: dom.HELEMENT;       -- source element e.g. in SELECTION_CHANGED it is new selected <option>, in MENU_ITEM_CLICK it is menu item (LI) element
    reason*: EVENT_REASON;   -- EVENT_REASON or EDIT_CHANGED_REASON - UI action causing change.
                             -- In case of custom event notifications this may be any
                             -- application specific value.                                                                                                                                    
    data*: val.VALUE;        -- auxiliary data accompanied with the event. E.g. FORM_SUBMIT event is using this field to pass collection of values.    
  END;

  TIMER_PARAMS* = RECORD(PARAMS)
    timerId*: Windows.PUINT;    -- timerId that was used to create timer by using HTMLayoutSetTimerEx
  END;

  METHOD_PARAMS* = RECORD(PARAMS)
    methodID*: Windows.UINT; -- see: #BEHAVIOR_METHOD_IDENTIFIERS
  END;

  SCRIPTING_METHOD_PARAMS* = RECORD(PARAMS)
    name*: Windows.PSTR; -- method name
    argv*: val.PVALUE;   -- vector of arguments
    argc*: Windows.UINT; -- argument count
    result*: val.VALUE;  -- return value
  END;

  TISCRIPT_METHOD_PARAMS* = RECORD(PARAMS)
    vm*: ti.Ptiscript_VM;
    tag*: ti.tiscript_value;     --< method id (symbol)
    result*: ti.tiscript_value ; --< return value
  END;
-- parameters are accessible through tiscript::args.

  DATA_ARRIVED_PARAMS* = RECORD(PARAMS)
    initiator*: dom.HELEMENT;    -- element intiator of HTMLayoutRequestElementData request,
    data*: Windows.PBYTE;        -- data buffer
    dataSize*: Windows.UINT;     -- size of data
    dataType*: Windows.UINT;     -- data type passed "as is" from HTMLayoutRequestElementData
    status*: Windows.UINT;       -- status = 0 (dataSize == 0) - unknown error.
                                 -- status = 100..505 - http response status, Note: 200 - OK!
                                 -- status > 12000 - wininet error code, see ERROR_INTERNET_*** in wininet.h
    uri*: Windows.PWSTR;         -- requested url
  END;



END Sciter_Behavior_Defs.
