<*+CHANGESYM*>
<*+M2EXTENSIONS*>
<*+O2EXTENSIONS*>
<* ALIGNMENT = "8" *>
MODULE Sciter_TIScriptAPI_Defs;

IMPORT 
  Windows, 
  SYSTEM;

TYPE
  tiscript_value* = SYSTEM.INT64;
  Ptiscript_VM*   = Windows.PVOID;

  Ptiscript_pvalue* = POINTER ["StdCall"] TO tiscript_pvalue;
  tiscript_pvalue* = RECORD 
    val*: tiscript_value;
    vm*: Ptiscript_VM;
    d1*: Windows.PVOID;
    d2*: Windows.PVOID;
  END;

  tiscript_stream* = POINTER ["C"] TO tiscript_stream_rec;

  tiscript_stream_input  * = PROCEDURE ["C"] (tag: tiscript_stream; VAR v: Windows.LONG): Windows.BOOL;
  tiscript_stream_output * = PROCEDURE ["C"] (tag: tiscript_stream;     v: Windows.LONG): Windows.BOOL;
  tiscript_stream_name   * = PROCEDURE ["C"] (tag: tiscript_stream): Windows.PWSTR;
  tiscript_stream_close  * = PROCEDURE ["C"] (tag: tiscript_stream);

  tiscript_stream_vtbl* = POINTER ["C"] TO RECORD
    input*:    tiscript_stream_input;
    output*:   tiscript_stream_output;
    get_name*: tiscript_stream_name;
    close*:    tiscript_stream_close;
  END;

  tiscript_stream_rec* = RECORD
    _vtbl*: tiscript_stream_vtbl;
  END;

  tiscript_method*      = PROCEDURE ["C"] (c: Ptiscript_VM): tiscript_value;
  tiscript_get_prop*    = PROCEDURE ["C"] (c: Ptiscript_VM; obj: tiscript_value): tiscript_value;
  tiscript_set_prop*    = PROCEDURE ["C"] (c: Ptiscript_VM; obj: tiscript_value; tv: tiscript_value);
  tiscript_get_item*    = PROCEDURE ["C"] (c: Ptiscript_VM; obj: tiscript_value; key: tiscript_value): tiscript_value;
  tiscript_set_item*    = PROCEDURE ["C"] (c: Ptiscript_VM; obj: tiscript_value; key: tiscript_value; value: tiscript_value);
  tiscript_iterator*    = PROCEDURE ["C"] (c: Ptiscript_VM; VAR index: tiscript_value; obj: tiscript_value): tiscript_value;
  tiscript_object_enum* = PROCEDURE ["C"] (c: Ptiscript_VM; key: tiscript_value; value: tiscript_value; tag: Windows.PVOID): Windows.BOOL;
  tiscript_finalizer*   = PROCEDURE ["C"] (c: Ptiscript_VM; obj: tiscript_value);
  tiscript_on_gc_copy*  = PROCEDURE ["C"] (instance_data: Windows.PVOID; new_self: tiscript_value);
  tiscript_callback*    = PROCEDURE ["C"] (c: Ptiscript_VM; rm: Windows.PVOID);

  tiscript_method_def*  = POINTER ["C"] TO RECORD
    dispatch*: Windows.PVOID; -- a.k.a. VTBL
    name*: Windows.PSTR;
    handler*: tiscript_method;
  	tag*: Windows.PVOID;
  END;

  tiscript_prop_def*    = POINTER ["C"] TO RECORD
    dispatch*: Windows.PVOID; -- a.k.a. VTBL
    name*: Windows.PSTR;
    getter*: tiscript_get_prop;
    setter*: tiscript_set_prop;
  	tag*: Windows.PVOID;
  END;

  tiscript_const_def* = POINTER ["C"] TO tiscript_const_def_base;
  tiscript_const_def_base* = RECORD
  END;

  tiscript_const_def_int* = RECORD(tiscript_const_def_base)
    name*: Windows.PSTR;
    i*: Windows.LONG;
    type*: Windows.UINT;
  END;

  tiscript_const_def_string* = RECORD(tiscript_const_def_base)
    name*: Windows.PSTR;
    str*: Windows.PSTR;
    type*: Windows.UINT;
  END;

  tiscript_const_def_float* = RECORD(tiscript_const_def_base)
    name*: Windows.PSTR;
    f*: LONGREAL;
    type*: Windows.UINT;
  END;

CONST
  TISCRIPT_CONST_INT    * = 0;
  TISCRIPT_CONST_FLOAT  * = 1;
  TISCRIPT_CONST_STRING * = 2;

TYPE
  tiscript_class_def = POINTER ["C"] TO RECORD
    name*: Windows.PCHAR;      -- having this name
    methods*:   POINTER TO ARRAY OF tiscript_method_def; -- with these methods
    props*:     POINTER TO ARRAY OF tiscript_prop_def;   -- with these properties
    consts*:    POINTER TO ARRAY OF tiscript_const_def;  -- with these constants (if any)
    get_item*:  tiscript_get_item;                       -- var v = obj[idx]
    set_item*:  tiscript_set_item;                       -- obj[idx] = v
    finalizer*: tiscript_finalizer;                      -- destructor of native objects
    iterator*:  tiscript_iterator;                       -- for(var el in collecton) handler
    on_gc_copy*: tiscript_on_gc_copy;                    -- called by GC to notify that 'self' is moved to new location  
    prototype*: tiscript_value;                          -- superclass, prototype for the class (or 0)*)
  END;


TYPE
  Ptiscript_native_interface* = POINTER ["StdCall"] TO tiscript_native_interface;
  tiscript_native_interface*  = RECORD 
    -- create new tiscript_VM [and make it current for the thread].
    create_vm*:          PROCEDURE ["C"] (features: Windows.UINT; heap_size: Windows.UINT; stack_size: Windows.UINT): Ptiscript_VM;
    -- destroy tiscript_VM
    destroy_vm*:         PROCEDURE ["C"] (pvm: Ptiscript_VM);
    -- invoke GC
    invoke_gc*:          PROCEDURE ["C"] (pvm: Ptiscript_VM);
    -- TODO set stdin, stdout and stderr for this tiscript_VM 
    set_std_streams*:    PROCEDURE ["C"] (pvm: Ptiscript_VM; input: Windows.PVOID; output: Windows.PVOID; error: Windows.PVOID);
    -- get tiscript_VM attached to the current thread
    get_current_vm*:     PROCEDURE ["C"] (): Ptiscript_VM;
    -- get global namespace (Object)
    get_global_ns*:      PROCEDURE ["C"] (pvm: Ptiscript_VM): tiscript_value;
    -- get current namespace (Object)
    get_current_ns*:     PROCEDURE ["C"] (pvm: Ptiscript_VM): tiscript_value;

    is_int*:             PROCEDURE ["C"] (v: tiscript_value): Windows.BOOL;
    is_float*:           PROCEDURE ["C"] (v: tiscript_value): Windows.BOOL;
    is_symbol*:          PROCEDURE ["C"] (v: tiscript_value): Windows.BOOL;
    is_string*:          PROCEDURE ["C"] (v: tiscript_value): Windows.BOOL;
    is_array*:           PROCEDURE ["C"] (v: tiscript_value): Windows.BOOL;
    is_object*:          PROCEDURE ["C"] (v: tiscript_value): Windows.BOOL;
    is_native_object*:   PROCEDURE ["C"] (v: tiscript_value): Windows.BOOL;
    is_function*:        PROCEDURE ["C"] (v: tiscript_value): Windows.BOOL;
    is_native_function*: PROCEDURE ["C"] (v: tiscript_value): Windows.BOOL;
    is_instance_of*:     PROCEDURE ["C"] (v, cls: tiscript_value): Windows.BOOL;
    is_undefined*:       PROCEDURE ["C"] (v: tiscript_value): Windows.BOOL;
    is_nothing*:         PROCEDURE ["C"] (v: tiscript_value): Windows.BOOL;
    is_null*:            PROCEDURE ["C"] (v: tiscript_value): Windows.BOOL;
    is_true*:            PROCEDURE ["C"] (v: tiscript_value): Windows.BOOL;
    is_false*:           PROCEDURE ["C"] (v: tiscript_value): Windows.BOOL;
    is_class*:           PROCEDURE ["C"] (pvm: Ptiscript_VM; v: tiscript_value): Windows.BOOL;
    is_error*:           PROCEDURE ["C"] (v: tiscript_value): Windows.BOOL;
    is_bytes*:           PROCEDURE ["C"] (v: tiscript_value): Windows.BOOL;

    get_int_value*:      PROCEDURE ["C"] (v: tiscript_value; VAR i: Windows.LONG): Windows.BOOL;
    get_float_value*:    PROCEDURE ["C"] (v: tiscript_value; VAR d: LONGREAL): Windows.BOOL;
    get_bool_value*:     PROCEDURE ["C"] (v: tiscript_value; VAR d: Windows.BOOL): Windows.BOOL;
    get_symbol_value*:   PROCEDURE ["C"] (v: tiscript_value; VAR d: Windows.PCHAR): Windows.BOOL;
    get_string_value*:   PROCEDURE ["C"] (v: tiscript_value; VAR data: Windows.PWSTR; VAR length: Windows.UINT): Windows.BOOL;
    get_bytes*:          PROCEDURE ["C"] (v: tiscript_value; VAR b: Windows.PBYTE; VAR blen: Windows.UINT): Windows.BOOL;

    nothing_value*:      PROCEDURE ["C"] (): tiscript_value; -- special value that designates "does not exist" result.
    undefined_value*:    PROCEDURE ["C"] (): tiscript_value; 
    null_value*:         PROCEDURE ["C"] (): tiscript_value; 
    bool_value*:         PROCEDURE ["C"] (v: Windows.BOOL): tiscript_value; 
    int_value*:          PROCEDURE ["C"] (v: Windows.LONG): tiscript_value; 
    float_value*:        PROCEDURE ["C"] (v: LONGREAL): tiscript_value; 
    string_value*:       PROCEDURE ["C"] (vm: Ptiscript_VM; text: Windows.PWSTR; text_length: Windows.UINT): tiscript_value; 
    symbol_value*:       PROCEDURE ["C"] (zstr: Windows.PCHAR): tiscript_value; 
    bytes_value*:        PROCEDURE ["C"] (vm: Ptiscript_VM; text: Windows.PBYTE; data_length: Windows.UINT): tiscript_value; 

    to_string*:          PROCEDURE ["C"] (vm: Ptiscript_VM; v: tiscript_value): tiscript_value; 

    -- define native class
    define_class*:       PROCEDURE ["C"] (vm: Ptiscript_VM; cls: tiscript_class_def; zns: tiscript_value): tiscript_value;

     -- object
    create_object*:      PROCEDURE ["C"] (vm: Ptiscript_VM; of_class: tiscript_value): tiscript_value;

    set_prop*:           PROCEDURE ["C"] (vm: Ptiscript_VM; obj, key, tv: tiscript_value): Windows.BOOL;
    get_prop*:           PROCEDURE ["C"] (vm: Ptiscript_VM; obj, key, tv: tiscript_value): tiscript_value;

    for_each_prop*:      PROCEDURE ["C"] (vm: Ptiscript_VM; obj: tiscript_value; cb: tiscript_object_enum; tag: Windows.PVOID);
    
    get_instance_data*:  PROCEDURE ["C"] (obj: tiscript_value): Windows.PVOID;
    set_instance_data*:  PROCEDURE ["C"] (obj: tiscript_value; data: Windows.PVOID);

     -- array
    create_array*:       PROCEDURE ["C"] (vm: Ptiscript_VM; of_size: Windows.UINT): tiscript_value;
    set_elem*:           PROCEDURE ["C"] (vm: Ptiscript_VM; obj: tiscript_value; idx: Windows.UINT; tv: tiscript_value): Windows.BOOL;
    get_elem*:           PROCEDURE ["C"] (vm: Ptiscript_VM; obj: tiscript_value; idx: Windows.UINT): tiscript_value;

    set_array_size*:     PROCEDURE ["C"] (vm: Ptiscript_VM; obj: tiscript_value; of_size: Windows.UINT): tiscript_value;
    get_array_size*:     PROCEDURE ["C"] (vm: Ptiscript_VM; obj: tiscript_value): Windows.UINT;

     -- eval
    eval*:               PROCEDURE ["C"] (vm: Ptiscript_VM; ns: tiscript_value; input: Windows.PVOID; template_mode: Windows.BOOL; 
                                          VAR retval: tiscript_value): Windows.BOOL;
    eval_string*:        PROCEDURE ["C"] (vm: Ptiscript_VM; ns: tiscript_value; script: Windows.PWSTR; script_length: Windows.UINT;
                                          VAR retval: tiscript_value): Windows.BOOL;

     -- call function (method)
    call*:               PROCEDURE ["C"] (vm: Ptiscript_VM; obj: tiscript_value; function: tiscript_value; VAR argv: tiscript_value; argn: Windows.UINT; 
                                          VAR retval: tiscript_value): Windows.BOOL;

     -- compiled bytecodes
    compile*:            PROCEDURE ["C"] (vm: Ptiscript_VM; input: Windows.PVOID; output_bytecodes: Windows.PVOID; template_mode: Windows.BOOL): Windows.BOOL;
    loadbc*:             PROCEDURE ["C"] (vm: Ptiscript_VM; input_bytecodes: Windows.PVOID): Windows.BOOL;

     -- throw error
    throw_error*:        PROCEDURE ["C"] (vm: Ptiscript_VM; error: Windows.PWSTR);

    -- arguments access
    get_arg_count*:      PROCEDURE ["C"] (vm: Ptiscript_VM): Windows.UINT;
    get_arg_n*:          PROCEDURE ["C"] (vm: Ptiscript_VM; n: Windows.UINT): tiscript_value;


     -- path here is global "path" of the object, something like
     -- "one"
     -- "one.two", etc.
    get_value_by_path*:  PROCEDURE ["C"] (vm: Ptiscript_VM; VAR v: tiscript_value; path: Windows.PSTR): Windows.BOOL;


    -- pins
    pin*:                PROCEDURE ["C"] (vm: Ptiscript_VM; pp: Ptiscript_pvalue);
    unpin*:              PROCEDURE ["C"] (pp: Ptiscript_pvalue);

     -- create native_function_value and native_property_value, 
     -- use this if you want to add native functions/properties in runtime to exisiting classes or namespaces (including global ns)
    native_function_value*: PROCEDURE ["C"] (vm: Ptiscript_VM; p: tiscript_method_def): tiscript_value;
    native_property_value*: PROCEDURE ["C"] (vm: Ptiscript_VM; p: tiscript_prop_def): tiscript_value;

     -- Schedule execution of the pfunc(prm) in the thread owning this VM.
     -- Used when you need to call scripting methods from threads other than main (GUI) thread
     -- It is safe to call tiscript functions inside the pfunc. 
     -- returns 'true' if scheduling of the call was accepted, 'false' when failure (VM has no dispatcher attached). 
    post*:               PROCEDURE ["C"] (vm: Ptiscript_VM; func: tiscript_callback; prm: Windows.PVOID): Windows.BOOL;

     -- Introduce alien VM to the host VM:
     -- Calls method found on "host_method_path" (if there is any) on the pvm_host
     -- notifying the host about other VM (alien) creation. Return value of script function "host_method_path" running in pvm_host is passed
     -- as a parametr of a call to function at "alien_method_path".
     -- One of possible uses of this function:
     -- Function at "host_method_path" creates async streams that will serve a role of stdin, stdout and stderr for the alien vm.
     -- This way two VMs can communicate with each other.
    introduce_vm*:       PROCEDURE ["C"] (vm_host: Ptiscript_VM; host_method_path: Windows.PSTR; pvm_alien: Ptiscript_VM; alien_method_path: Windows.PSTR): Windows.UINT;

    set_remote_std_streams*: PROCEDURE ["C"] (vm: Ptiscript_VM; input, output, error: Windows.PVOID): Windows.BOOL;

     -- support of multi-return values from native fucntions, n here is a number 1..64
    set_nth_retval*:     PROCEDURE ["C"] (vm: Ptiscript_VM; n: Windows.LONG; ns: tiscript_value): Windows.BOOL;

    -- returns number of props in object, elements in array, or bytes in byte array.
    get_length*:         PROCEDURE ["C"] (vm: Ptiscript_VM; obj: tiscript_value): Windows.LONG;
    -- for( var val in coll ) {...}
    get_next*:           PROCEDURE ["C"] (vm: Ptiscript_VM; VAR obj, pos, val: tiscript_value): Windows.BOOL;

     -- for( var (key,val) in coll ) {...}
    get_next_key_value*: PROCEDURE ["C"] (vm: Ptiscript_VM; VAR obj, pos, key, val: tiscript_value): Windows.BOOL;

     -- associate extra data pointer with the VM
    set_extra_data*:     PROCEDURE ["C"] (vm: Ptiscript_VM; data: Windows.PVOID): Windows.BOOL;
    get_extra_data*:     PROCEDURE ["C"] (vm: Ptiscript_VM): Windows.PVOID;
  END;


END Sciter_TIScriptAPI_Defs.