<*+CHANGESYM*>
<*+M2EXTENSIONS*>
<*+O2EXTENSIONS*>
<* ALIGNMENT = "8" *>
MODULE Sciter_Value_Defs;

IMPORT 
  Windows, SYSTEM;

TYPE
  VALUE_RESULT = SYSTEM.INT32;
CONST
  HV_OK_TRUE           * = -1;
  HV_OK                * =  0;
  HV_BAD_PARAMETER     * =  1;
  HV_INCOMPATIBLE_TYPE * =  2;

TYPE
  PVALUE * = POINTER ["StdCall"] TO VALUE;
  VALUE * = RECORD
    t: Windows.UINT;
    u: Windows.UINT;
    d: SYSTEM.CARD64;
  END;
  
  FLOAT_VALUE * = LONGREAL;

CONST 
  T_UNDEFINED  * = 0;
  T_NULL       * = 1;
  T_BOOL       * = 2;
  T_INT        * = 3;
  T_FLOAT      * = 4;
  T_STRING     * = 5;
  T_DATE       * = 6;   (*  INT64 - contains a 64-bit value representing the number of 100-nanosecond intervals since January 1, 1601 (UTC), a.k.a. FILETIME on Windows *)
  T_CURRENCY   * = 7;   (*  INT64 - 14.4 fixed number. E.g. dollars = int64 / 10000;  *)
  T_LENGTH     * = 8;   (*  length units, value is int or float, units are VALUE_UNIT_TYPE *)
  T_ARRAY      * = 9;
  T_MAP        * = 10;
  T_FUNCTION   * = 11;
  T_BYTES      * = 12;   (*  sequence of bytes - e.g. image data *)
  T_OBJECT     * = 13;   (*  scripting object proxy (TISCRIPT/SCITER) *)
  T_DOM_OBJECT * = 14;   (*  DOM object (CSSS!), use get_object_data to get HELEMENT  *)

TYPE
  VALUE_TYPE * = SYSTEM.INT32;

CONST 
  UT_EM     * = 1;   (* height of the element's font.  *)
  UT_EX     * = 2;   (* height of letter 'x'  *)
  UT_PR     * = 3;   (* % *)
  UT_SP     * = 4;   (* %% "springs", a.k.a. flex units *)
  reserved1 * = 5;
  reserved2 * = 6;
  UT_PX     * = 7;   (* pixels *)
  UT_IN     * = 8;   (* inches (1 inch   = 2.54 centimeters).  *)
  UT_CM     * = 9;   (* centimeters.  *)
  UT_MM     * = 10;  (* millimeters.  *)
  UT_PT     * = 11;  (* points (1 point = 1/72 inches).  *)
  UT_PC     * = 12;  (* picas (1 pica = 12 points).  *)
  UT_DIP    * = 13;
  reserved3 * = 14;
  UT_COLOR  * = 15;  (*  color in int *)
  UT_URL    * = 16;  (*  url in string *)
  UT_SYMBOL * = 0FFFFH;   (*  for T_STRINGs designates symbol string ( so called NAME_TOKEN - CSS or JS identifier ) *)

TYPE
  VALUE_UNIT_TYPE * = SYSTEM.INT32;

TYPE
  
  VALUE_UNIT_TYPE_DATE * = SYSTEM.SET32;

CONST
  DT_HAS_DATE    * = VALUE_UNIT_TYPE_DATE {0}; -- = 0x01, date contains date portion
  DT_HAS_TIME    * = VALUE_UNIT_TYPE_DATE {1}; -- = 0x02, date contains time portion HH:MM
  DT_HAS_SECONDS * = VALUE_UNIT_TYPE_DATE {2}; -- = 0x04, date contains time and seconds HH:MM:SS
  DT_UTC         * = VALUE_UNIT_TYPE_DATE {4}; -- = 0x10, T_DATE is known to be UTC. Otherwise it is local date/time

TYPE 
  VALUE_UNIT_TYPE_OBJECT * = SYSTEM.INT32;
                                                     
CONST
  UT_OBJECT_ARRAY    * = 0; -- type T_OBJECT of type Array
  UT_OBJECT_OBJECT   * = 1; -- type T_OBJECT of type Object
  UT_OBJECT_CLASS    * = 2; -- type T_OBJECT of type Type (class or namespace)
  UT_OBJECT_NATIVE   * = 3; -- type T_OBJECT of native Type with data slot (LPVOID)
  UT_OBJECT_FUNCTION * = 4; -- type T_OBJECT of type Function
  UT_OBJECT_ERROR    * = 5; -- type T_OBJECT of type Error

TYPE 
  KeyValueCallback * = PROCEDURE ["StdCall"] (param: Windows.PVOID; VAR key: VALUE; VAR value: VALUE): Windows.BOOL;

TYPE 
  VALUE_STRING_CVT_TYPE * = Windows.UINT;

CONST
  CVT_SIMPLE       * = 0; -- simple conversion of terminal values 
  CVT_JSON_LITERAL * = 1; -- json literal parsing/emission 
  CVT_JSON_MAP     * = 2; -- json parsing/emission, it parses as if token '{' already recognized 


PROCEDURE ["StdCall"] / ValueInit*(VAR value: VALUE): Windows.UINT;
PROCEDURE ["StdCall"] / ValueClear*(VAR value: VALUE): Windows.UINT;
PROCEDURE ["StdCall"] / ValueCompare*(VAR value1: VALUE; VAR val2: VALUE): Windows.UINT;
PROCEDURE ["StdCall"] / ValueCopy*(VAR dst: VALUE; VAR src: VALUE): Windows.UINT;
PROCEDURE ["StdCall"] / ValueIsolate*(VAR dst: VALUE): Windows.UINT;
PROCEDURE ["StdCall"] / ValueType*(VAR value: VALUE; VAR type: VALUE_TYPE; VAR units: VALUE_UNIT_TYPE): Windows.UINT;
PROCEDURE ["StdCall"] / ValueStringData*(VAR value: VALUE; VAR Chars: Windows.PWSTR; VAR NumChars: Windows.UINT): Windows.UINT;
PROCEDURE ["StdCall"] / ValueStringDataSet*(VAR value: VALUE; chars: Windows.PWSTR; numChars: Windows.UINT; units: VALUE_UNIT_TYPE): Windows.UINT;
PROCEDURE ["StdCall"] / ValueIntData*(VAR value: VALUE; VAR Data: Windows.LONG): Windows.UINT;
PROCEDURE ["StdCall"] / ValueIntDataSet*(VAR value: VALUE; data: Windows.LONG; type: VALUE_TYPE; units: VALUE_UNIT_TYPE): Windows.UINT;
PROCEDURE ["StdCall"] / ValueInt64Data*(VAR value: VALUE; VAR data: SYSTEM.INT64): Windows.UINT;
PROCEDURE ["StdCall"] / ValueInt64DataSet*(VAR value: VALUE; data: SYSTEM.INT64; type: VALUE_TYPE; units: VALUE_UNIT_TYPE): Windows.UINT;
PROCEDURE ["StdCall"] / ValueFloatData*(VAR value: VALUE; VAR Data: FLOAT_VALUE): Windows.UINT;
PROCEDURE ["StdCall"] / ValueFloatDataSet*(VAR value: VALUE; data: FLOAT_VALUE; type: VALUE_TYPE; units: VALUE_UNIT_TYPE): Windows.UINT;
PROCEDURE ["StdCall"] / ValueBinaryData*(VAR value: VALUE; VAR Bytes: Windows.PBYTE; VAR nBytes: Windows.UINT): Windows.UINT;
PROCEDURE ["StdCall"] / ValueBinaryDataSet*(VAR value: VALUE; Bytes:  Windows.PBYTE; nBytes: Windows.UINT; type: VALUE_TYPE; units: VALUE_UNIT_TYPE): Windows.UINT;
PROCEDURE ["StdCall"] / ValueElementsCount*(VAR value: VALUE; VAR n: Windows.LONG): Windows.UINT;
PROCEDURE ["StdCall"] / ValueNthElementValue*(VAR value: VALUE; n: Windows.LONG; VAR retval: VALUE): Windows.UINT;
PROCEDURE ["StdCall"] / ValueNthElementValueSet*(VAR value: VALUE; n: Windows.LONG; VAR val_to_set: VALUE): Windows.UINT;
PROCEDURE ["StdCall"] / ValueNthElementKey*(VAR value: VALUE; n: Windows.LONG; VAR retval: VALUE): Windows.UINT;
PROCEDURE ["StdCall"] / ValueEnumElements*(VAR value: VALUE; enum: KeyValueCallback; param: Windows.PVOID): Windows.UINT;
PROCEDURE ["StdCall"] / ValueSetValueToKey*(VAR value: VALUE; VAR key: VALUE; VAR val_to_set: VALUE): Windows.UINT;
PROCEDURE ["StdCall"] / ValueGetValueOfKey*(VAR value: VALUE; VAR key: VALUE; VAR retval: VALUE): Windows.UINT;
PROCEDURE ["StdCall"] / ValueToString*(VAR value: VALUE; how: VALUE_STRING_CVT_TYPE): Windows.UINT;
PROCEDURE ["StdCall"] / ValueFromString*(VAR value: VALUE; str: Windows.PWSTR; strLength: Windows.UINT; how: VALUE_STRING_CVT_TYPE): Windows.UINT;
PROCEDURE ["StdCall"] / ValueInvoke*(VAR value: VALUE; VAR this: VALUE; argc: Windows.UINT; VAR argv: VALUE; VAR retval: VALUE; url: Windows.PWSTR): Windows.UINT;

END Sciter_Value_Defs.